/*
 * Copyright (C) 2021 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package android.car.cts.app;

import android.app.Activity;
import android.car.Car;
import android.car.hardware.power.CarPowerManager;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import androidx.annotation.Nullable;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import javax.annotation.concurrent.GuardedBy;

/**
 * To test car power policy:
 *     <pre class="prettyprint">
 *         adb shell am force-stop android.car.cts.app
 *         adb shell am start -n android.car.cts.app/.PowerPolicyTestActivity \
 *              --es "powerpolicy" "testcase,action[,data]"
 *         - testcase: suite | 1 | 2 | 3 | 4 | 5 | 6
 *         - action:   start | end | dumpstate | dumppolicy | applypolicy | closefile
 *         - data:     policyId
 *     </pre>
 */
public final class PowerPolicyTestActivity extends Activity {
    private static final String TAG = PowerPolicyTestActivity.class.getSimpleName();
    private static final String SHARED_DATA_FILE = "/storage/emulated/obb/PowerPolicyData.txt";

    private Car mCarApi;
    private final Object mLock = new Object();
    @GuardedBy("mLock")
    private CarPowerManager mPowerManager;
    private PrintWriter mPrintWriter;

    @Nullable
    public CarPowerManager getPowerManager() {
        synchronized (mLock) {
            return mPowerManager;
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        Log.d(TAG, "onNewIntent");
        Bundle extras = intent.getExtras();
        if (extras == null) {
            Log.w(TAG, "onNewIntent: empty extras");
            return;
        }
        PowerPolicyTestCommand cmd = PowerPolicyTestClient.parseCommand(extras);
        if (cmd == null) {
            Log.w(TAG, "onNewIntent: invalid power policy test command");
            return;
        }
        cmd.setCar(mCarApi);
        cmd.setPrintWriter(mPrintWriter);
        PowerPolicyTestClient.handleCommand(cmd);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initCarApi();
        Context ctx = getApplicationContext();
        try {
            mPrintWriter = new PrintWriter(new File(ctx.getFilesDir(), SHARED_DATA_FILE));
        } catch (IOException e) {
            Log.e(TAG, "onCreate: failed to open PowerPolicyData.txt");
            mPrintWriter = null;
        }

        Log.d(TAG, "onCreate");
        onNewIntent(getIntent());
    }

    private void initCarApi() {
        if (mCarApi != null && mCarApi.isConnected()) {
            mCarApi.disconnect();
            mCarApi = null;
        }
        mCarApi = Car.createCar(this, null, Car.CAR_WAIT_TIMEOUT_WAIT_FOREVER,
                (Car car, boolean ready) -> {
                    initManagers(car, ready);
                });
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy");
        if (mCarApi != null) {
            mCarApi.disconnect();
        }
        super.onDestroy();
    }

    private void initManagers(Car car, boolean ready) {
        synchronized (mLock) {
            if (ready) {
                mPowerManager = (CarPowerManager) car.getCarManager(
                        android.car.Car.POWER_SERVICE);
                Log.d(TAG, "initManagers() completed");
            } else {
                mPowerManager = null;
                Log.wtf(TAG, "initManagers() set to be null");
            }
        }
    }
}
